﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Documents.Parsing;

namespace WpfApplication3
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window
	{
		public MainWindow()
		{
			InitializeComponent();

			var vbLanguage = VisualBasicLanguage.Instance;

			// Clone the VB grammar so we can modify it.
			var customGrammar = vbLanguage.Grammar.Clone();

			// Define the result keyword in the default lexer state and mark it as a reserved keyword.
			// Insert it at the beginning of the symbols collection because "Result" will be matched by
			// both ResultKeyword and IdentifierToken now and ResultKeyword should have higher priority.
			var resultKeyword = customGrammar.LexerStates.DefaultLexerState.Symbols.Insert(0,
				"ResultKeyword",
				"Result",
				TerminalSymbolComparison.LiteralIgnoreCase);
			resultKeyword.LanguageElement = LanguageElement.ReservedKeyword;

			// The _identifierResolved hidden symbol is used in all places an identifier could be used. 
			// It is currently defined as an alternation of all things that could be considered an identifier.
			// Add the ResultKeyword as another type of identifier so it will be allowed in all places 
			// identifiers are allowed.
			var identifierResolved = customGrammar.NonTerminalSymbols["_identifierResolved"];
			var identifierTypes = (AlternationSyntaxRule)identifierResolved.Rule;
			identifierTypes.Rules.Add(resultKeyword);

			// Create a language based on our cloned grammar and copy over the services from the VB language. 
			// The one we are really interested in is the indentation service provider.
			var customLanguage = new CustomLanguage(customGrammar);
			foreach (var serviceKey in vbLanguage.ServicesManager.GetRegisteredServiceKeys())
				customLanguage.ServicesManager.RegisterService(serviceKey, vbLanguage.ServicesManager.GetService(serviceKey));

			foreach (var serviceType in vbLanguage.ServicesManager.GetRegisteredServiceTypes())
				customLanguage.ServicesManager.RegisterService(serviceType, vbLanguage.ServicesManager.GetService(serviceType));

			// Set the language on the syntax editor.
			this.SyntaxEditor.Document.Language = customLanguage;
		}
	}
}
